//simpsquosc.cpp - simplistic square wave oscillator (complete w/aliasing)

#include <math.h>
#include "simpsquosc.hpp"

simpsquosc::simpsquosc(float sr, float freq)
{
  double y;

  frequency = freq;
  samplerate = sr;

  accumulator = 0.0f;
  y = 65536.0/(double)samplerate;
  increment = y*(double)frequency;
}

void simpsquosc::fakeconstructor(float sr, float freq)
{
  double y;

  frequency = freq;
  samplerate = sr;

  accumulator = 0.0f;
  y = 65536.0/(double)samplerate;
  increment = y*(double)frequency;
}

float simpsquosc::getsample(float freq)
{
  double y;

  if(freq != frequency)
  {
	  frequency = freq;
	  y = 65536.0/(double)samplerate;
	  increment = y*(double)frequency;
  }

  if(frequency <= 0.0f)
	  y = 0.0;
  else if(accumulator < 32768.0)
  {
	accumulator += increment;
    y = 1.0;
  }
  else if(accumulator < 65536.0)
  {
    accumulator += increment;
    y = -1.0;
  }
  else
  {
	accumulator = fmod(accumulator, 65536.0);
    y = 1.0;
  }

  return (float)y;
}

void simpsquosc::resetPhase()
{
	accumulator = 0.0;
}

void simpsquosc::resetPhaseTo(float val)
{
	accumulator = (val * 65536.0) - increment;
}